"""
This file is part of dispy project.
See http://dispy.sourceforge.net for details.
"""

__author__ = "Giridhar Pemmasani (pgiri@yahoo.com)"
__email__ = "pgiri@yahoo.com"
__copyright__ = "Copyright 2015, Giridhar Pemmasani"
__contributors__ = []
__maintainer__ = "Giridhar Pemmasani (pgiri@yahoo.com)"
__license__ = "MIT"
__url__ = "http://dispy.sourceforge.net"

__all__ = ['GetHandler', 'node_status']

import sys
import os
import threading
import json
import dispy
if sys.version_info.major > 2:
    from http.server import BaseHTTPRequestHandler
else:
    from BaseHTTPServer import BaseHTTPRequestHandler

# variables in this block can/should be updated by the client
# directory where cluster.css, jquery.js etc., are stored:
DocumentRoot = os.path.join(os.path.dirname(dispy.__file__), 'data')
cluster_poll_sec = 10 # cluster information is updated at this interval
jobs_submitted = 0 # this must be incremented when a job is submitted by dispy client

cluster = {}
jobs_done = 0
cluster_lock = threading.Lock()

class GetHandler(BaseHTTPRequestHandler):
    # def log_message(self, format, *args):
    #     return

    def do_GET(self):
        if self.path == '/data':
            global cluster, cluster_lock
            cluster_lock.acquire()
            nodes = cluster.values()
            cluster = {}
            cluster_lock.release()
            self.send_response(200)
            self.send_header('Content-Type', 'application/json; charset=utf-8')
            self.end_headers()
            self.wfile.write(json.dumps({'jobs':{'submitted':jobs_submitted, 'done':jobs_done},
                                         'nodes':[node.__dict__ for node in nodes]}).encode())
            return
        elif self.path == '/' or self.path == '/cluster.html':
            self.send_response(200)
            self.send_header('Content-type', 'text/html')
            self.end_headers()
            path = os.path.join(DocumentRoot, 'cluster.html')
            try:
                f = open(path)
                html = f.read()
                self.wfile.write((html % {'TIMEOUT':str(cluster_poll_sec * 1000)}).encode())
                f.close()
            except:
                print('Could not read/send "%s"' % path)
            return
        else:
            path = self.path[1:]
            try:
                if path in ('jquery.js', 'cluster.css'):
                    f = open(os.path.join(DocumentRoot, path))
                    self.send_response(200)
                    self.send_header('Content-type',
                                     'text/javascript' if path.endswith('.js') else 'text/css')
                    self.end_headers()
                    self.wfile.write(f.read().encode())
                    f.close()
            except:
                print('Could not read/send "%s"' % self.path)
                import traceback
                print(traceback.format_exc())
            return
           
def node_status(node, status, job):
    global cluster, jobs_done
    cluster_lock.acquire()
    cluster[node.ip_addr] = node
    cluster_lock.release()
    if status == dispy.DispyNode.JobDone:
        jobs_done += 1
